@@ -1,22 +1,39 @@
 import operator
 import sqlalchemy as sa
-from sqlalchemy_utils import primary_keys, identity
+from sqlalchemy_utils import get_primary_keys, identity
 from .utils import tx_column_name, end_tx_column_name
 
 
-def eq(tuple_):
-    return tuple_[0] == tuple_[1]
-
-
 def parent_identity(obj_or_class):
+    if (
+        isinstance(obj_or_class, sa.sql.selectable.Alias) or
+        isinstance(obj_or_class, sa.Table)
+    ):
+        return tuple(
+            column
+            for column in obj_or_class.c
+            if column.name != 'transaction_id'
+            and column.primary_key
+        )
     return tuple(
-        getattr(obj_or_class, column.name)
-        for column in primary_keys(obj_or_class)
-        if column.name != tx_column_name(obj_or_class)
+        getattr(obj_or_class, column_key)
+        for column_key in get_primary_keys(obj_or_class).keys()
+        if column_key != tx_column_name(obj_or_class)
     )
 
 
-class HistoryObjectFetcher(object):
+def eqmap(callback, iterable):
+    for a, b in zip(*map(callback, iterable)):
+        yield a == b
+
+
+def parent_criteria(obj, class_=None):
+    if class_ is None:
+        class_ = obj.__class__
+    return eqmap(parent_identity, (class_, obj))
+
+
+class VersionObjectFetcher(object):
     def __init__(self, manager):
         self.manager = manager
 
@@ -43,16 +60,7 @@ class HistoryObjectFetcher(object):
         """
         return self.next_query(obj).first()
 
-    def parent_identity_correlation(self, obj):
-        return map(
-            eq,
-            zip(
-                parent_identity(obj.__class__),
-                parent_identity(obj)
-            )
-        )
-
-    def _transaction_id_subquery(self, obj, next_or_prev='next'):
+    def _transaction_id_subquery(self, obj, next_or_prev='next', alias=None):
         if next_or_prev == 'next':
             op = operator.gt
             func = sa.func.min
@@ -60,24 +68,30 @@ class HistoryObjectFetcher(object):
             op = operator.lt
             func = sa.func.max
 
-        alias = sa.orm.aliased(obj)
+        if alias is None:
+            alias = sa.orm.aliased(obj)
+            table = alias.__table__
+            attrs = alias
+        else:
+            table = alias.original
+            attrs = alias.c
         query = (
             sa.select(
                 [func(
-                    getattr(alias, tx_column_name(obj))
+                    getattr(attrs, tx_column_name(obj))
                 )],
-                from_obj=[alias.__table__]
+                from_obj=[table]
             )
             .where(
                 sa.and_(
                     op(
-                        getattr(alias, tx_column_name(obj)),
+                        getattr(attrs, tx_column_name(obj)),
                         getattr(obj, tx_column_name(obj))
                     ),
-                    *map(eq, zip(parent_identity(alias), parent_identity(obj)))
+                    *eqmap(parent_identity, (alias, obj))
                 )
             )
-            .correlate(alias.__table__)
+            .correlate(table)
         )
         return query
 
@@ -96,7 +110,7 @@ class HistoryObjectFetcher(object):
                     self._transaction_id_subquery(
                         obj, next_or_prev=next_or_prev
                     ),
-                    *self.parent_identity_correlation(obj)
+                    *parent_criteria(obj)
                 )
             )
         )
@@ -121,9 +135,7 @@ class HistoryObjectFetcher(object):
         query = (
             sa.select([subquery], from_obj=[obj.__table__])
             .where(
-                sa.and_(
-                    *map(eq, zip(identity(obj.__class__), identity(obj)))
-                )
+                sa.and_(*eqmap(identity, (obj.__class__, obj)))
             )
             .order_by(
                 getattr(obj.__class__, tx_column_name(obj))
@@ -132,7 +144,7 @@ class HistoryObjectFetcher(object):
         return query
 
 
-class SubqueryFetcher(HistoryObjectFetcher):
+class SubqueryFetcher(VersionObjectFetcher):
     def previous_query(self, obj):
         """
         Returns the query that fetches the previous version relative to this
@@ -148,7 +160,7 @@ class SubqueryFetcher(HistoryObjectFetcher):
         return self._next_prev_query(obj, 'next')
 
 
-class ValidityFetcher(HistoryObjectFetcher):
+class ValidityFetcher(VersionObjectFetcher):
     def next_query(self, obj):
         """
         Returns the query that fetches the next version relative to this
@@ -160,16 +172,10 @@ class ValidityFetcher(HistoryObjectFetcher):
             session.query(obj.__class__)
             .filter(
                 sa.and_(
-                    getattr(
-                        obj.__class__,
-                        tx_column_name(obj)
-                    )
+                    getattr(obj.__class__, tx_column_name(obj))
                     ==
-                    getattr(
-                        obj,
-                        end_tx_column_name(obj)
-                    ),
-                    *self.parent_identity_correlation(obj)
+                    getattr(obj, end_tx_column_name(obj)),
+                    *parent_criteria(obj)
                 )
             )
         )
@@ -185,13 +191,10 @@ class ValidityFetcher(HistoryObjectFetcher):
             session.query(obj.__class__)
             .filter(
                 sa.and_(
-                    getattr(
-                        obj.__class__,
-                        end_tx_column_name(obj)
-                    )
+                    getattr(obj.__class__, end_tx_column_name(obj))
                     ==
                     getattr(obj, tx_column_name(obj)),
-                    *self.parent_identity_correlation(obj)
+                    *parent_criteria(obj)
                 )
             )
         )
