@@ -3,7 +3,7 @@ import sqlalchemy as sa
 
 class TableBuilder(object):
     """
-    TableBuilder handles the building of history tables based on parent
+    TableBuilder handles the building of version tables based on parent
     table's structure and versioning configuration options.
     """
     def __init__(
@@ -17,15 +17,30 @@ class TableBuilder(object):
         self.model = model
 
     def option(self, name):
-        return self.manager.option(self.model, name)
+        try:
+            return self.manager.option(self.model, name)
+        except TypeError:
+            return self.manager.options[name]
 
     @property
     def table_name(self):
         """
-        Returns the history table name for current parent table.
+        Returns the version table name for current parent table.
         """
         return self.option('table_name') % self.parent_table.name
 
+    @property
+    def parent_columns(self):
+        for column in self.parent_table.c:
+            if (
+                self.model and
+                self.manager.is_excluded_column(self.model, column)
+            ):
+                continue
+            if not self.model and column in self.manager.options['exclude']:
+                continue
+            yield column
+
     @property
     def reflected_columns(self):
         """
@@ -40,28 +55,9 @@ class TableBuilder(object):
 
         transaction_column_name = self.option('transaction_column_name')
 
-        for column in self.parent_table.c:
-            if self.manager.is_excluded_column(self.model, column):
-                continue
-
+        for column in self.parent_columns:
             column_copy = self.reflect_column(column)
             columns.append(column_copy)
-            if (
-                self.option('track_property_modifications') and
-                not column.primary_key
-            ):
-                columns.append(
-                    sa.Column(
-                        column_copy.name + self.option('modified_flag_suffix'),
-                        sa.Boolean,
-                        key=(
-                            column_copy.key +
-                            self.option('modified_flag_suffix')
-                        ),
-                        default=False,
-                        nullable=False
-                    )
-                )
 
         # When using join table inheritance each table should have own
         # transaction column.
@@ -107,7 +103,8 @@ class TableBuilder(object):
         return sa.Column(
             self.option('operation_type_column_name'),
             sa.SmallInteger,
-            nullable=False
+            nullable=False,
+            index=True
         )
 
     @property
@@ -120,6 +117,7 @@ class TableBuilder(object):
             self.option('transaction_column_name'),
             sa.BigInteger,
             primary_key=True,
+            index=True,
             autoincrement=False  # This is needed for MySQL
         )
 
@@ -131,23 +129,29 @@ class TableBuilder(object):
         """
         return sa.Column(
             self.option('end_transaction_column_name'),
-            sa.BigInteger
+            sa.BigInteger,
+            index=True
         )
 
+    @property
+    def columns(self):
+        data = self.reflected_columns
+        data.append(self.transaction_column)
+        if self.option('strategy') == 'validity':
+            data.append(self.end_transaction_column)
+        data.append(self.operation_type_column)
+        return data
+
     def __call__(self, extends=None):
         """
-        Builds history table.
+        Builds version table.
         """
-        items = []
-        if extends is None:
-            items.extend(self.reflected_columns)
-            items.append(self.transaction_column)
-            if self.option('strategy') == 'validity':
-                items.append(self.end_transaction_column)
-            items.append(self.operation_type_column)
+        columns = self.columns if extends is None else []
+
+        self.manager.plugins.after_build_version_table_columns(self, columns)
         return sa.schema.Table(
             extends.name if extends is not None else self.table_name,
             self.parent_table.metadata,
-            *items,
+            *columns,
             extend_existing=extends is not None
         )
