@@ -4,23 +4,25 @@ import itertools as it
 import os
 import warnings
 import sqlalchemy as sa
-from six import PY3
-from pytest import mark
 from sqlalchemy import create_engine
 from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy.orm import sessionmaker
-from sqlalchemy_continuum import make_versioned, versioning_manager
-from sqlalchemy_continuum.ext.flask import (
-    versioning_manager as flask_versioning_manager
+from sqlalchemy_continuum import (
+    ClassNotVersioned,
+    version_class,
+    make_versioned,
+    versioning_manager,
+    remove_versioning
+)
+from sqlalchemy_continuum.transaction import TransactionFactory
+from sqlalchemy_continuum.plugins import (
+    TransactionMetaPlugin,
+    TransactionChangesPlugin
 )
 
-
 warnings.simplefilter('error', sa.exc.SAWarning)
 
 
-make_versioned(options={'strategy': 'subquery'})
-
-
 class QueryPool(object):
     queries = []
 
@@ -37,14 +39,14 @@ def log_sql(
     QueryPool.queries.append(statement)
 
 
-# @mark.skipif("PY3 and os.environ.get('DB') == 'mysql'")
 class TestCase(object):
     versioning_strategy = 'subquery'
     transaction_column_name = 'transaction_id'
     end_transaction_column_name = 'end_transaction_id'
-    track_property_modifications = False
-    store_data_at_delete = False
     composite_pk = False
+    plugins = [TransactionChangesPlugin(), TransactionMetaPlugin()]
+    transaction_cls = TransactionFactory()
+    user_cls = None
 
     @property
     def options(self):
@@ -53,40 +55,41 @@ class TestCase(object):
             'strategy': self.versioning_strategy,
             'transaction_column_name': self.transaction_column_name,
             'end_transaction_column_name': self.end_transaction_column_name,
-            'track_property_modifications': self.track_property_modifications,
-            'store_data_at_delete': self.store_data_at_delete
         }
 
-    def setup_class(cls):
-        versioning_manager.options['versioning'] = True
-        flask_versioning_manager.options['versioning'] = False
+    def get_dns_from_driver(self, driver):
+        if driver == 'postgres':
+            return 'postgres://postgres@localhost/sqlalchemy_continuum_test'
+        elif driver == 'mysql':
+            return 'mysql+pymysql://travis@localhost/sqlalchemy_continuum_test'
+        elif driver == 'sqlite':
+            return 'sqlite:///:memory:'
+        else:
+            raise Exception('Unknown driver given: %r' % driver)
 
     def setup_method(self, method):
-        adapter = os.environ.get('DB', 'postgres')
-        if adapter == 'postgres':
-            dns = 'postgres://postgres@localhost/sqlalchemy_continuum_test'
-        elif adapter == 'mysql':
-            dns = 'mysql+pymysql://travis@localhost/sqlalchemy_continuum_test'
-        elif adapter == 'sqlite':
-            dns = 'sqlite:///:memory:'
-        else:
-            raise Exception('Unknown driver given: %r' % adapter)
+        self.Model = declarative_base()
+        make_versioned(options=self.options)
 
-        self.engine = create_engine(dns)
+        driver = os.environ.get('DB', 'sqlite')
+        versioning_manager.plugins = self.plugins
+        versioning_manager.transaction_cls = self.transaction_cls
+        versioning_manager.user_cls = self.user_cls
+
+        self.engine = create_engine(self.get_dns_from_driver(driver))
         # self.engine.echo = True
         self.connection = self.engine.connect()
-        self.Model = declarative_base()
 
         self.create_models()
 
         sa.orm.configure_mappers()
 
         if hasattr(self, 'Article'):
-            self.ArticleHistory = self.Article.__versioned__['class']
+            self.ArticleVersion = version_class(self.Article)
         if hasattr(self, 'Tag'):
             try:
-                self.TagHistory = self.Tag.__versioned__['class']
-            except (AttributeError, KeyError):
+                self.TagVersion = version_class(self.Tag)
+            except ClassNotVersioned:
                 pass
         self.create_tables()
 
@@ -100,9 +103,9 @@ class TestCase(object):
         self.Model.metadata.drop_all(self.connection)
 
     def teardown_method(self, method):
+        remove_versioning()
         QueryPool.queries = []
         versioning_manager.reset()
-        versioning_manager.uow.reset()
 
         self.session.close_all()
         self.session.expunge_all()
@@ -135,8 +138,6 @@ class TestCase(object):
 
 setting_variants = {
     'versioning_strategy': ['subquery', 'validity'],
-    'store_data_at_delete': [True, False],
-    'track_property_modifications': [True, False],
     'transaction_column_name': ['transaction_id', 'tx_id'],
     'end_transaction_column_name': ['end_transaction_id', 'end_tx_id']
 }
